/*!
  * vue-promised v2.0.3
  * (c) 2021 Eduardo San Martin Morote
  * @license MIT
  */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vueDemi = require('vue-demi');

/**
 * Returns the state of a Promise and observes the Promise if it's a Ref to
 * automatically update the state
 *
 * @param promise Ref of a Promise or raw Promise
 * @param pendingDelay optional delay to wait before displaying pending
 */
function usePromise(promise, pendingDelay = 200) {
    const isPending = vueDemi.ref(false);
    const isDelayElapsed = vueDemi.ref(false);
    const error = vueDemi.ref();
    const data = vueDemi.ref();
    let timerId;
    vueDemi.watch(() => vueDemi.unref(promise), (newPromise) => {
        isPending.value = true;
        error.value = null;
        if (!newPromise) {
            data.value = null;
            isPending.value = true;
            if (timerId)
                clearTimeout(timerId);
            timerId = null;
            return;
        }
        if (vueDemi.unref(pendingDelay) > 0) {
            isDelayElapsed.value = false;
            if (timerId)
                clearTimeout(timerId);
            timerId = setTimeout(() => {
                isDelayElapsed.value = true;
            }, Number(vueDemi.unref(pendingDelay)));
        }
        else {
            isDelayElapsed.value = true;
        }
        newPromise
            .then((newData) => {
            // ensure we are dealing with the same promise
            if (newPromise === vueDemi.unref(promise)) {
                data.value = newData;
                isPending.value = false;
            }
        })
            .catch((err) => {
            // ensure we are dealing with the same promise
            if (newPromise === vueDemi.unref(promise)) {
                error.value = err;
                isPending.value = false;
            }
        });
    }, { immediate: true });
    return { isPending, isDelayElapsed, error, data };
}

const Promised = vueDemi.defineComponent({
    name: 'Promised',
    props: {
        promise: {},
        // validator: p =>
        //   p && typeof (p as any).then === 'function' && typeof (p as any).catch === 'function',
        pendingDelay: {
            type: [Number, String],
            default: 200,
        },
    },
    setup(props, { slots }) {
        const propsAsRefs = vueDemi.toRefs(props);
        const promiseState = vueDemi.reactive(usePromise(propsAsRefs.promise, propsAsRefs.pendingDelay));
        return () => {
            if ('combined' in slots) {
                return slots.combined(promiseState);
            }
            const [slotName, slotData] = promiseState.error
                ? ['rejected', promiseState.error]
                : !promiseState.isPending
                    ? ['default', promiseState.data]
                    : promiseState.isDelayElapsed
                        ? ['pending', promiseState.data]
                        : [null];
            return slotName && slots[slotName](slotData);
        };
    },
});

exports.Promised = Promised;
exports.usePromise = usePromise;
