/*!
  * vue-promised v2.0.3
  * (c) 2021 Eduardo San Martin Morote
  * @license MIT
  */
import { ref, watch, unref, defineComponent, toRefs, reactive, isVue3, warn } from 'vue-demi';

/**
 * Returns the state of a Promise and observes the Promise if it's a Ref to
 * automatically update the state
 *
 * @param promise Ref of a Promise or raw Promise
 * @param pendingDelay optional delay to wait before displaying pending
 */
function usePromise(promise, pendingDelay = 200) {
    const isPending = ref(false);
    const isDelayElapsed = ref(false);
    const error = ref();
    const data = ref();
    let timerId;
    watch(() => unref(promise), (newPromise) => {
        isPending.value = true;
        error.value = null;
        if (!newPromise) {
            data.value = null;
            isPending.value = true;
            if (timerId)
                clearTimeout(timerId);
            timerId = null;
            return;
        }
        if (unref(pendingDelay) > 0) {
            isDelayElapsed.value = false;
            if (timerId)
                clearTimeout(timerId);
            timerId = setTimeout(() => {
                isDelayElapsed.value = true;
            }, Number(unref(pendingDelay)));
        }
        else {
            isDelayElapsed.value = true;
        }
        newPromise
            .then((newData) => {
            // ensure we are dealing with the same promise
            if (newPromise === unref(promise)) {
                data.value = newData;
                isPending.value = false;
            }
        })
            .catch((err) => {
            // ensure we are dealing with the same promise
            if (newPromise === unref(promise)) {
                error.value = err;
                isPending.value = false;
            }
        });
    }, { immediate: true });
    return { isPending, isDelayElapsed, error, data };
}

const Promised = defineComponent({
    name: 'Promised',
    props: {
        promise: {},
        // validator: p =>
        //   p && typeof (p as any).then === 'function' && typeof (p as any).catch === 'function',
        pendingDelay: {
            type: [Number, String],
            default: 200,
        },
    },
    setup(props, { slots }) {
        const propsAsRefs = toRefs(props);
        const promiseState = reactive(usePromise(propsAsRefs.promise, propsAsRefs.pendingDelay));
        return () => {
            if ('combined' in slots) {
                return slots.combined(promiseState);
            }
            const [slotName, slotData] = promiseState.error
                ? ['rejected', promiseState.error]
                : !promiseState.isPending
                    ? ['default', promiseState.data]
                    : promiseState.isDelayElapsed
                        ? ['pending', promiseState.data]
                        : [null];
            if ((process.env.NODE_ENV !== 'production') && slotName && !slots[slotName]) {
                (isVue3 ? warn : console.warn)(`(vue-promised) Missing slot "${slotName}" in Promised component. This will fail in production.`);
                return null;
            }
            return slotName && slots[slotName](slotData);
        };
    },
});

export { Promised, usePromise };
